
function isRequired (paramName) {
  const callerName = isRequired.caller.name;
  let msg = 'Missing required parameter';
  msg += paramName ? ` "${paramName}"` : '';
  msg += callerName ? ` in function "${callerName}"` : '';
  // noinspection UnnecessaryLocalVariableJS
  const PARAM_REQUIRED = msg;
  throw new Error(PARAM_REQUIRED);
}

function isFunction (fn) {
  const toString = Object.prototype.toString;
  return !!(
    // Duck typing
    fn &&
    fn.constructor &&
    fn.call &&
    fn.apply &&
    // Strong typing
    toString.call(fn) === '[object Function]'
  );
}

function isWindow (obj) {
  const toString = Object.prototype.toString;
  return !!(
    // Duck typing
    obj &&
    obj === obj.window &&
    obj.document &&
    obj.location &&
    // Strong typing
    toString.call(obj) === '[object Window]'
  );
}

function isJQuery ($) {
  return !!(
    // Duck typing
    $ &&
    $.fn &&
    $.fn.jquery &&
    $.fn.extend &&
    // Strong typing
    isFunction($)
  );
}

function initJQuery (
  jQuery = isRequired('jQuery'),
  window = isRequired('window'),
  extendJQuery
) {
  // Check window
  if (!isWindow(window)) {
    throw new Error('Window object is required');
  }

  // Return if window.$ was initialized
  if (
    window.$ &&
    isJQuery(window.$)
  ) {
    return window.$;
  }

  // Init jQuery
  const $ = jQuery(window);

  // Extend jQuery
  if (isFunction(extendJQuery)) {
    extendJQuery($);
  }

  return $;
}

module.exports = initJQuery;
